@@ -429,7 +429,7 @@ class Figure():
 
     actorDest = self.findActor(smartParent)
     if not actorDest:
-      logging.warning("Figure[%s] has not Actor[%s] required by Magnet[%s]", self.getName(), smartParent, baseProp.getName())
+      logging.warning("Figure[%s] has no Actor[%s] required by Magnet[%s]", self.getPrintName(), smartParent, baseProp.getName())
       return C_UNKNOWN
 
     # Remove SmartParent attributs
@@ -461,7 +461,7 @@ class Figure():
           
           dparm.addVOD(vop)
       else:
-        logging.warning("Figure[%s] has no Actor[%s] for Magnet[%s]", self.getName(), targetName, deformerProp.getName())
+        logging.warning("Figure[%s] has no Actor[%s] for Magnet[%s]", self.getPrintName(), targetName, deformerProp.getName())
         ret = min(ret, C_FAIL)
 
     return ret
