@@ -152,7 +152,7 @@ def RemoveQuotes(s):
   return s
 
 def WBoolLine(fw, pfx, name, bval):
-  fw.write('{0:s}{1:s} {2:s}\n'.format(pfx, name, ("1" if bval else "0")))
+  fw.write(f"{pfx}{name} {'1' if bval else '0'}\n")
 
 class PoserConst(object):
   C_EX_NOTCALL = 'pypos3d.pftk.error.NotCallable'
@@ -176,10 +176,13 @@ class PoserConst(object):
   C_DATE_CHANGED = 0x1002
 
   # Bad figure index 
-  C_BAD_INDEX = -128
+  C_BAD_INDEX         = -128
+  C_BAD_QUALNAME      = -129
 
   # Actor not found in search operations 
-  C_ACTOR_NOTFOUND = -2
+  C_ACTOR_NOTFOUND    = -2
+  C_MATERIAL_NOTFOUND = -130
+  C_NODE_NOTFOUND     = -131
 
   # Already existing thing 
   C_EXISTS = 0x0F00
