@@ -20,7 +20,7 @@ Created on 23 nov. 2020
 '''
 import logging
 
-from pypos3d.wftk.WFBasic import Point3d, TexCoord2f
+from pypos3d.wftk.WFBasic import Point3d, TexCoord2f, C_FACE_TEXT
 from pypos3d.wftk.WaveGeom import WaveGeom
 
 from pypos3d.propslim import MX_UNBOUND, MX_ALL_MASK, MX_VALID_FLAG, MX_PERVERTEX, MX_MAX_BINDING
@@ -368,9 +368,19 @@ class MxStdModel(object):
       try:
         for grp in geom.getGroups():
           vertIdxTbl = grp.vertIdx
-          tvertIdxTbl = grp.tvertIdx
+          tvertIdxTbl = [-1] * len(grp.vertIdx) # grp.tvertIdx
+          
+          # I must admit that this kind of Vertex --> Texture mapping is FALSE!!
+          for faceno, fao in enumerate(grp.matIdx):
+            if fao.fAttr & C_FACE_TEXT:
+              si = grp.getFaceStartIdx(faceno)
+              ei = grp.getFaceLastIdx(faceno)
+              tvertIdxTbl[si:ei] = fao.tvertIdx
+
           for i,v in enumerate(vertIdxTbl):
             mapVT[v] = tvertIdxTbl[i]
+          
+            
       except IndexError:
         logging.warning("Geom[%s] has bad texture index", geom.getName())
         hasTexture = False
