@@ -585,6 +585,10 @@ class GeomGroup(object):
 
   def invertFaceOrder(self):
     ''' Invert all faces order (ie. revert 'auto' normals)
+    
+    Return
+    ------
+    self for convenience
     '''
     tmpVert = [] # array.array('l')
 
@@ -599,6 +603,7 @@ class GeomGroup(object):
         
     self.vertIdx[:] = array.array('l', tmpVert)
       
+    return self
 
     
 
@@ -618,7 +623,7 @@ class GeomGroup(object):
     # Verify Vertex indexes
     for vi in self.vertIdx:
       if (vi<0) or (vi>=sizeVertexList):
-        print('Group[{0:s}] Bad Vertex Index: {1:d}'.format(name, vi))
+        print(f'Group[{name}] Bad Vertex Index: {vi}')
         res = C_ERROR
         
     # Check faces counts
@@ -630,13 +635,13 @@ class GeomGroup(object):
       if fao.fAttr & C_FACE_TEXT:
         for vti in fao.tvertIdx: # [startIdx:endIdx]:
           if (vti<0) or (vti>=sizeTexList):
-            print('Group[{0:s}].face[{:d}] Bad Texture Index List: {1:d}'.format(name, faceno, vti))
+            print(f'Group[{name}].face[{faceno}] Bad Texture Index List: {vti}')
             res = C_ERROR
 
       if fao.fAttr & C_FACE_NORM:
         for vti in fao.normIdx: #[startIdx:endIdx]:
           if (vti<0) or (vti>=sizeNormList):
-            print('Group[{0:s}].face[{:d}] Bad Normal Index : {1:d}'.format(name, faceno, vti))
+            print(f'Group[{name}].face[{faceno}] Bad Normal Index: {vti}')
             res = C_ERROR
       
       nbv = self.getFaceLastIdx(faceno) - self.getFaceStartIdx(faceno)
@@ -647,7 +652,7 @@ class GeomGroup(object):
       else:
         nbfo+=1
 
-    print('Group[{0:s}] has {1:d} Triangles, {2:d} Quad, {3:d} others'.format(name, nbf3, nbf4, nbfo))
+    print(f'Group[{name}] has {nbf3} Triangles, {nbf4} Quad, {nbf4} others')
     
     return res
     
@@ -917,7 +922,7 @@ class GeomGroup(object):
                   nvti = IndexAdd(self.texList, TexCoord2f(ngrp.texList[vti]))
                   mapTVert[vti] = nvti
             
-              fao.tvertIdx[i] = nvti
+                fao.tvertIdx[i] = nvti
     
             if fao.fAttr & C_FACE_NORM:
               for i,vti in enumerate(fao.normIdx):
@@ -1059,7 +1064,7 @@ class GeomGroup(object):
       swaped = False
     
     if WFBasic.PYPOS3D_TRACE:
-      for i in range(0,nbl0):self.geom.addMaterial('DebugMat'+str(i))
+      for i in range(0,nbl0):self.geom.addMaterial(f'DebugMat{i}')
       self.geom.curGroup = self
     
     hasTexture = min( [ l.hasTexture for l in l0 ] ) and min( [ l.hasTexture for l in l1 ] )
@@ -1092,7 +1097,7 @@ class GeomGroup(object):
     
     while oneAgain:
       if WFBasic.PYPOS3D_TRACE:
-        self.geom.selectMaterial('DebugMat'+str(i))
+        self.geom.selectMaterial(f'DebugMat{i}')
           
       e0 = l0[i]
       e1 = l1[j]
@@ -1536,6 +1541,9 @@ class GeomGroup(object):
     # Remap Group's vertex indexes with the new ones
     self.vertIdx[:] = array.array('l', [ mapVert[vi] for vi in self.vertIdx ] )
     
+    # Invert face orders
+    self.invertFaceOrder()
+    
     return self
     
     
@@ -1550,10 +1558,10 @@ class GeomGroup(object):
     faceno : int
       Face to remove (if >0)
 
-    materialName : str
+    materialName : str, default None
       Name of the material. Use the first face with this material
       
-    axisOrder : str, defaul 'xyz'
+    axisOrder : str, default 'xyz'
       Rotation order between 'xyz', 'yzx' and 'zxy'
 
     Returns
@@ -1578,41 +1586,12 @@ class GeomGroup(object):
       return C_ERROR, None
 
     ret, vr = C_OK, None
-    #vxtab = self.geomGroup.getFaceVertexBy(tor[5:], raiseExcept=False)
     vxtab = self.getFaceVertex(faceno)    
     if vxtab and len(vxtab)>=3:
-      # fcenter, p1, p2 = vxtab[:3]
-      #
-      # # Try to be more accurate on non triangular faces
-      # if len(vxtab)>3:
-      #   fcenter = Point3d.barycentre(vxtab)
-      #   p1, p2 = vxtab[0], vxtab[1]
-      #   v0 = Vector3d.AB(fcenter, p1)
-      #   v1 = Vector3d.AB(fcenter, p2)
-      #   # Compute a first normale to get a sign reference
-      #   n0 = v0.cross(v1)
-      #
-      #   # Look for a point with a maximum (and better) absolute 'sinus'
-      #   bestnorm, maxs = n0, n0.norme2()
-      #   for p in vxtab[2:]:
-      #     v2 = Vector3d.AB(fcenter, p)
-      #     pv = v0.cross(v2)
-      #
-      #     n = pv.norme2()
-      #     if n>maxs:
-      #       p2 = p
-      #       bestnorm = pv
-      #       maxs = n
-      #
-      #   # If the best point generates a reserved normale, swap points order
-      #   if n0.dot(bestnorm) < 0.0:
-      #     p1, p2 = p2, p1
-      
-      # Perform the normal computation  
-      # vr = Vector3d( Point3d.triangle_orientation(fcenter, p1, p2, AxisOrder=axisOrder, inDegrees=True) )
+      # Perform the orientation computation  
       vr = Point3d.face_orientation(vxtab, AxisOrder=AxisOrder, inDegrees=True)              
     else:  
-      ret = C_FAIL # , msg = C_FAIL, f"Orientation error for [{refGrp.getName()}]: Missing Face {tor}"
+      ret = C_FAIL
             
     return ret, vr
     
