@@ -159,6 +159,16 @@ def SArcCos(a,b):
   else:
     return 0.0
 
+def SArcSin(a,b):
+  ''' Perform asin( a / b ) with rounding effects '''
+  if math.fabs(b)>FEPSILON:
+    r = a / b
+    r = -1.0 if r<-1.0 else (1.0 if r>1.0 else r)
+    return math.asin(r)
+  else:
+    return 1.0
+
+
 # Used as base Object for Vector3d
 class Point3d(object):
   ''' Simple 3D point class (mutable). '''
@@ -225,7 +235,15 @@ class Point3d(object):
   def get(self,i): return self.x if i==0 else self.y if i==1 else self.z
 
   def __getitem__(self, i):
-    return self.x if i==0 else self.y if i==1 else self.z
+    if i==0:
+      return self.x 
+    elif i==1:
+      return self.y 
+    elif i==2:
+      return self.z
+    else:
+      # Fix 20210626: Avoid Infinte loop in *()
+      raise IndexError()
 
   def __eq__(self, other):
     ''' Compare two Point3d.
@@ -419,6 +437,10 @@ class Point3d(object):
 
     t2 : Point3d 
       The tuple to be added
+  
+    Return
+    ------
+    self
     '''
     self.x = t2.x + s*t1.x
     self.y = t2.y + s*t1.y
@@ -427,7 +449,7 @@ class Point3d(object):
     return self
 
   def __str__(self, *args, **kwargs):
-    return 'P({0:g},{1:g},{2:g})'.format(self.x, self.y, self.z)
+    return 'P({0:.6g},{1:.6g},{2:.6g})'.format(self.x, self.y, self.z)
     
   def poserPrint(self):
     return '{0: 11.8f} {1: 11.8f} {2: 11.8f}'.format(self.x, self.y, self.z)
@@ -472,6 +494,27 @@ class Point3d(object):
 
   @classmethod
   def triangle_orientation(cls, p0, p1, p2, AxisOrder='xyz', inDegrees=False):
+    ''' Compute the set of rotations (RotX, RotY, RotZ) that put the first axis of
+    AxisOrder on the normal of the triangle.
+
+    Parameters
+    ----------
+    p1, p2, p3 : Point3d
+      Triangle definition
+
+    AxisOrder : str, defaul 'xyz'
+      Rotation order between 'xyz', 'yzx' and 'zxy'
+
+    inDegrees : bool, default False
+      if True returns the orientation in degrees
+      else orientation is in radians
+
+    Return
+    ------
+    Vector3d
+      The result rotation vector
+    '''
+    
     n = Point3d.triangle_normal(p0, p1, p2)
     
     rotx, roty, rotz = 0.0,0.0,0.0
@@ -480,15 +523,30 @@ class Point3d(object):
       rotz = math.asin(-n.x)
       cz   = math.cos(rotz)
       rotx = SArcCos(n.y, cz)
+
+      # Check Third Equation
+      if math.fabs(n.z - math.sin(rotx)*cz) > FEPSILON:
+        # Sign mismatch --> Use other arccos solution
+        rotx = -rotx
+    
     elif AxisOrder=='xyz':
-      roty = math.asin(n.z)
+      roty = - math.asin(n.z)
       cy   = math.cos(roty)
       rotz = SArcCos(n.x, cy)
+
+      # Check Third Equation
+      if math.fabs(n.y - math.sin(rotz)*cy) > FEPSILON:
+        # Sign mismatch --> Use other arccos solution
+        rotz = -rotz
+    
     else: # 'zxy'
-      rotx = math.asin(-n.y)
+      rotx = - math.asin(n.y)
       cx   = math.cos(rotx)
       roty = SArcCos(n.z, cx)
-    
+      
+      if math.fabs(n.x - math.sin(roty)*cx) > FEPSILON:
+        roty = -roty # SArcSin(n.x, cx)
+        
     # Convert Result un degree
     if inDegrees:
       rotx = math.degrees(rotx)
@@ -497,6 +555,68 @@ class Point3d(object):
 
     return (rotx, roty, rotz)
 
+
+  @classmethod
+  def face_orientation(cls, vxtab:'list of Point3d', AxisOrder='xyz', inDegrees=False):
+    ''' Compute the orientation of a face according to an Axis order (xyz, yzx, zxy).
+    The face is defined by a list of Point3d
+    The orientation is the set of rotations that aligns the first axis of the
+    axis order on the face normal vector.
+    
+    Parameters
+    ----------
+    vxtab : list of Point3d
+      Face definition. Shall have 3 points at least
+
+    axisOrder : str, defaul 'xyz'
+      Rotation order between 'xyz', 'yzx' and 'zxy'
+
+    inDegrees : bool, default False
+      if True returns the orientation in degrees
+      else orientation is in radians
+
+    Return
+    ------
+    Vector3d
+      None if vxtab is empty or too small
+    '''
+
+    if not vxtab or len(vxtab)<3:
+      return None
+    
+    fcenter, p1, p2 = vxtab[:3]
+    
+    # Try to be more accurate on non triangular faces
+    if len(vxtab)>3:
+      fcenter = Point3d.barycentre(vxtab)
+      p1, p2 = vxtab[0], vxtab[1]
+      v0 = Vector3d.AB(fcenter, p1)
+      v1 = Vector3d.AB(fcenter, p2)
+      # Compute a first normale to get a sign reference
+      n0 = v0.cross(v1)
+      
+      # Look for a point with a maximum (and better) absolute 'sinus'
+      bestnorm, maxs = n0, n0.norme2()
+      for p in vxtab[2:]:
+        v2 = Vector3d.AB(fcenter, p)
+        pv = v0.cross(v2)
+        
+        n = pv.norme2()
+        if n>maxs:
+          p2 = p
+          bestnorm = pv
+          maxs = n
+
+      # If the best point generates a reserved normale, swap points order
+      if n0.dot(bestnorm) < 0.0:
+        p1, p2 = p2, p1
+    
+    # Perform the orientation computation  
+    vr = Vector3d( Point3d.triangle_orientation(fcenter, p1, p2, AxisOrder=AxisOrder, inDegrees=inDegrees) )              
+      
+    return vr
+
+
   @classmethod
   def barycentre(cls, iterPt:'iterable of Point3d'):
     '''Compute the iso-barycenter of a set (or list) of Point3d.
@@ -519,6 +639,61 @@ class Point3d(object):
     else:
       return None
 
+
+  @classmethod
+  def rotateAxis(cls, r, p, AxisOrder='xyz'):
+    ''' Perform rotations defined in parameter 'r' according to 
+    order given by AxisOrder.
+    
+    Parameters
+    ----------
+    r : Vector3d
+      Rotation vector (RotX, RotY, RotZ)
+
+    AxisOrder : str, defaul 'xyz'
+      Rotation order between 'xyz', 'yzx' and 'zxy'
+
+    Return
+    ------
+    Point3d
+      Rotated element
+    '''
+    theta = r.x*math.pi/180.0
+    c, s = math.cos(theta), math.sin(theta)
+    MRotX = [ \
+       [ 1.0, 0.0, 0.0], \
+       [ 0.0,   c, - s ], \
+       [ 0.0,   s,   c ], \
+       ]
+  
+    theta = r.y*math.pi/180.0
+    c, s = math.cos(theta), math.sin(theta)
+    MRotY = [ \
+       [   c, 0.0,   s ], \
+       [ 0.0, 1.0, 0.0 ], \
+       [ - s, 0.0,   c ], \
+       ]
+
+    theta = r.z*math.pi/180.0
+    c, s = math.cos(theta), math.sin(theta)
+    MRotZ = [ \
+      [   c, - s, 0.0], \
+      [   s,   c, 0.0 ], \
+      [ 0.0, 0.0, 1.0 ], \
+      ]
+
+    if AxisOrder=='xyz':
+      m = MxProd(MRotY, MRotX)
+      m = MxProd(MRotZ, m)
+    elif AxisOrder=='yzx':
+      m = MxProd(MRotZ, MRotY)
+      m = MxProd(MRotX, m)
+    else: # zxy
+      m = MxProd(MRotX, MRotZ)
+      m = MxProd(MRotY, m)
+
+    return p.Lin33(m)
+
   # End of Class Point3d ======================================================
 
 
@@ -529,7 +704,22 @@ class Vector3d(Point3d):
     super(Vector3d, self).__init__(x,y,z)
 
   def __str__(self, *args, **kwargs):
-    return 'V({0:g},{1:g},{2:g})'.format(self.x, self.y, self.z)
+    return 'V({0:.6g},{1:.6g},{2:.6g})'.format(self.x, self.y, self.z)
+
+  @classmethod
+  def AB(cls, a:'Point3d', b:'Point3d', normalize=False):
+    '''Convenience method to create a Vector3d from 2 Point3d.
+    
+    Parameters
+    ----------
+    a: First Point
+    b: Second Point
+    
+    Return
+    ------
+    a Vector3d 
+    '''
+    return Vector3d( b.x-a.x, b.y-a.y, b.z-a.z ).normalize() if normalize else Vector3d( b.x-a.x, b.y-a.y, b.z-a.z )
   # End of Class Vector3d ======================================================
     
 
