@@ -1177,7 +1177,7 @@ class WaveGeom(object):
     sidx = set(grp.vertIdx)
     lidx = list(sidx)
     lidx.sort()
-    ngrp.vertIdx = [ bisect.bisect(lidx, vi)-1 for vi in grp.vertIdx ]
+    ngrp.vertIdx = array.array('l', [ bisect.bisect(lidx, vi)-1 for vi in grp.vertIdx ])
     nCoordList = [ Point3d(self.coordList[vi]) for vi in lidx ]
     
     # Remap Texture Coordinates          
